# Part 5: AI 핵심 라이브러리 마스터하기 (5주차)

**⬅️ 이전 시간: [Part 4: 객체 지향 프로그래밍(OOP)](./part_4_object_oriented_programming.md)** | **➡️ 다음 시간: [Part 6: 머신러닝 모델링과 평가](./part_6_machine_learning.md)**

---
## 📜 실습 코드 바로가기
- **`part_5_ai_core_libraries.py`**: [바로가기](./source_code/part_5_ai_core_libraries.py)
  - 본 파트에서 다루는 NumPy, Pandas, Matplotlib의 핵심 기능 예제 코드를 직접 실행하고 수정해볼 수 있습니다.

---

지난 4주간 우리는 파이썬이라는 언어 자체에 대한 이해와 객체 지향 설계 능력을 길렀습니다. 이제부터는 이 강력한 언어를 날개 삼아, 데이터의 세계를 본격적으로 탐험할 시간입니다.
2부에서는 데이터 분석과 머신러닝 프로젝트의 실질적인 '주연' 배우들인 NumPy, Pandas, Scikit-learn 등의 핵심 라이브러리 사용법을 깊이 있게 학습합니다.

---

## ⭐️ 2부의 시작: 핵심 라이브러리 관계도 (The Big Picture)

본격적으로 각 라이브러리를 배우기 전에, 이들이 데이터 분석 및 머신러닝 프로젝트에서 어떻게 함께 사용되는지에 대한 전체적인 그림을 이해하는 것이 중요합니다.

```mermaid
graph TD
    subgraph "데이터 수집 및 전처리"
        A["<b>Pandas (DataFrame)</b><br/>- CSV/Excel/DB에서 데이터 로드<br/>- 결측치 처리, 필터링, 그룹화, 병합"]
    end

    subgraph "고성능 수치 연산"
        B["<b>NumPy (ndarray)</b><br/>- DataFrame에서 변환<br/>- 벡터/행렬 연산, 수학 함수"]
    end

    subgraph "머신러닝 모델링 & 평가"
        C["<b>Scikit-learn (Estimator)</b><br/>- 모델 학습(fit), 예측(predict)<br/>- 모델 평가, 하이퍼파라미터 튜닝"]
    end

    subgraph "데이터 시각화"
        D["<b>Matplotlib & Seaborn</b><br/>(Figure, Axes)<br/>- 데이터 탐색(EDA)<br/>- 모델 결과 시각화"]
    end

    A -->|"데이터 정제"| A
    A -->|"df.values, df['column']"| B
    B -->|"모델 입력"| C
    A -->|"df.plot()"| D
    B -->|"plt.plot(array)"| D
    C -->|"결과 분석"| D
```
- **시작은 [Pandas](./glossary.md#pandas)**: 대부분의 프로젝트는 Pandas를 사용해 다양한 소스(CSV 등)로부터 데이터를 불러와 `DataFrame`으로 만드는 것에서 시작합니다.
- **계산을 위해 [NumPy](./glossary.md#numpy)로**: 정제된 데이터는 고성능 수치 계산을 위해 NumPy 배열(`ndarray`)로 변환되어 머신러닝 모델의 입력으로 준비됩니다.
- **[Scikit-learn](./glossary.md#scikit-learn)으로 모델링**: NumPy 배열로 변환된 데이터를 사용하여 Scikit-learn으로 모델을 학습하고 예측합니다.
- **모든 과정에서 시각화**: Pandas와 NumPy 데이터는 [Matplotlib/Seaborn](./glossary.md#matplotlib--seaborn)을 통해 시각화되어 데이터의 특징을 파악(EDA)하거나 모델의 성능을 분석하는 데 사용됩니다.

이 흐름을 염두에 두고 각 라이브러리를 학습하면, 지식이 파편적으로 남지 않고 하나의 큰 그림으로 연결될 것입니다.

---
---

# 1부: 데이터 과학자의 연장통 - NumPy & Pandas (5주차)

### 🎯 5주차 학습 목표
- **[NumPy](./glossary.md#numpy)**를 사용하여 고성능 과학 계산 및 다차원 배열을 다룹니다.
- **[Pandas](./glossary.md#pandas)**를 사용하여 표 형식의 데이터를 자유자재로 읽고, 선택하고, 필터링하고, 그룹화하고, 병합합니다.

---

## 1일차(월): NumPy - 고성능 수치 계산의 초석
- **NumPy(Numerical Python)**: 파이썬에서 과학 계산을 위한 가장 기본적인 패키지. [`ndarray`](./glossary.md#ndarray)라는 효율적인 다차원 배열 객체를 중심으로, 벡터, 행렬 등 고성능 수치 연산을 위한 방대한 함수들을 제공합니다.

#### 💡 쉽게 이해하기: NumPy 배열은 '계란판', 파이썬 리스트는 '장바구니'
> - **파이썬 리스트**: 뭐든 담을 수 있는 만능 '장바구니'입니다. 사과, 책, 신발 등 종류에 상관없이 담을 수 있어 유연하지만, 내용물이 제각각이라 전체를 한 번에 계산하기는 어렵고 느립니다.
> - **NumPy 배열**: 오직 한 종류(숫자)만 담을 수 있는 '계란판'과 같습니다. 모든 칸의 모양과 크기가 같아, '모든 계란에 도장 찍기' 같은 작업을 기계로 한 번에 처리할 수 있습니다. 이런 **균일성**과 **규칙성** 덕분에 NumPy는 엄청나게 빠른 속도로 숫자 계산을 할 수 있습니다. AI와 데이터 과학에서 대규모 데이터를 다룰 때 NumPy가 필수인 이유입니다.

### 1. `ndarray` 생성 및 연산
```python
import numpy as np

# 파이썬 리스트로부터 배열 생성
py_list = [[1, 2, 3], [4, 5, 6]]
np_array = np.array(py_list)
print("Shape:", np_array.shape) # (2, 3)

# 배열 연산: for문 없이 모든 요소에 대해 한 번에 연산
result = np_array * 2 + 1
print("배열 연산 결과:\n", result)

# 브로드캐스팅: 서로 다른 크기의 배열 간에도 연산이 가능하도록 확장
matrix = np.array([[1, 2, 3], [4, 5, 6]]) # shape: (2, 3)
vector = np.array([10, 20, 30])           # shape: (3,)
result_broadcast = matrix + vector
print("브로드캐스팅 결과:\n", result_broadcast)
```

### 2. 인덱싱과 슬라이싱
```python
arr = np.arange(10) # [0, 1, 2, 3, 4, 5, 6, 7, 8, 9]

# 인덱싱 및 슬라이싱
print("3번째 원소:", arr[2])
print("3~5번째 원소:", arr[2:5])

# 조건(Boolean) 인덱싱: 특정 조건을 만족하는 원소만 추출하는 강력한 기능
print("3보다 큰 원소:", arr[arr > 3])
print("짝수만 추출:", arr[arr % 2 == 0])
```

---
## 2일차(화): Pandas - '슈퍼 엑셀' DataFrame의 등장
- **Pandas**: [`DataFrame`](./glossary.md#dataframe)(2차원 테이블)과 [`Series`](./glossary.md#series)(1차원 배열) 자료구조를 사용하여, 엑셀이나 DB 테이블처럼 데이터를 직관적으로 다루게 해주는 라이브러리. 데이터 전처리에 없어서는 안 될 필수 도구입니다.

### 1. `Series`와 `DataFrame`
```python
import pandas as pd

# DataFrame 생성
data = {'제품': ['사과', '바나나', '체리', '딸기'],
        '가격': [1000, 500, 2000, 1500],
        '재고': [10, 20, 15, 5]}
df = pd.DataFrame(data)
print("DataFrame:\n", df)

# Series: DataFrame의 한 열
prices = df['가격']
print("\nSeries (가격):\n", prices)
```

### 2. 데이터 읽고 쓰기 및 기본 탐색
```python
# df.to_csv("fruits.csv", index=False, encoding="utf-8-sig") # CSV 파일로 저장
# df_from_csv = pd.read_csv("fruits.csv") # CSV 파일 읽기

print("\n--- head() ---")
print(df.head())      # 맨 위 5개 행
print("\n--- info() ---")
df.info()             # 각 열의 정보와 결측치 확인
print("\n--- describe() ---")
print(df.describe())  # 숫자형 열의 기술 통계량
```

---
## 3일차(수): Pandas - 데이터 선택과 필터링의 기술

### 1. `.loc` vs `.iloc`: 이름과 순서로 선택하기
> - **`.loc` (Label-based)**: 학생의 **'이름'**으로 자리를 찾는 방식 (`df.loc[행_이름, 열_이름]`)
> - **`.iloc` (Integer-based)**: 학생이 앉아있는 **'자리 번호'**로 찾는 방식 (`df.iloc[행_번호, 열_번호]`)

```python
# loc: 1번 인덱스 행의 '제품', '재고' 열
print(df.loc[1, ['제품', '재고']])

# iloc: 0번 행의 0, 2번 열
print(df.iloc[0, [0, 2]])
```

### 2. 조건으로 필터링하기
```python
# 가격이 1000원 이상인 제품
print("\n가격 1000원 이상:\n", df[df['가격'] >= 1000])

# 가격 1000원 미만이고(and) 재고 10개 미만인 제품
# & (and), | (or), ~ (not)
print("\n가격 1000원 미만 AND 재고 10개 미만:\n", df[(df['가격'] < 1000) & (df['재고'] < 10)])
```
> 🚫 **`SettingWithCopyWarning` 주의!**: 값을 변경할 때는 `.loc`를 사용해 `df.loc[행조건, 열이름] = 값` 형태로 한 번에 접근해야 의도치 않은 오류를 막을 수 있습니다. 경고는 "원본을 바꿀지, 복사본을 바꿀지 애매하니 명확히 해달라"는 신호입니다.

---
## 4일차(목): Pandas - 데이터 분해와 조립 (`groupby`)
`groupby()`는 데이터를 특정 기준으로 그룹화하여 각 그룹에 대한 집계(합계, 평균 등)를 수행하는 강력한 기능입니다.

#### 💡 쉽게 이해하기: `groupby`는 '반별 성적 평균 계산' (Split-Apply-Combine)
> `groupby`는 '반별 성적을 계산하는 선생님'과 같습니다.
> 1.  **Split (나누기)**: 전체 학생 명단을 '1반', '2반', '3반'으로 나눕니다 (`df.groupby('반')`).
> 2.  **Apply (적용하기)**: 각 반별로 '평균 점수 계산'이라는 동일한 작업을 수행합니다 (`.agg('mean')`).
> 3.  **Combine (합치기)**: 각 반의 평균 점수를 모아 하나의 최종 성적표로 합칩니다.

```python
data = {'팀': ['A', 'B', 'A', 'B', 'A'], '선수': ['손흥민', '김민재', '황희찬', '이강인', '조규성'], '득점': [12, 1, 8, 5, 10]}
df_player = pd.DataFrame(data)

# '팀' 별로 그룹화하여 득점의 합계, 평균, 선수 수 계산
team_agg = df_player.groupby('팀')['득점'].agg(['sum', 'mean', 'count'])
print(team_agg)
```

---
## 5일차(금): Pandas - 여러 데이터 합치고 모양 바꾸기

### 1. 데이터 병합: `pd.merge()`
> `merge`는 '학생 명단'과 '도서관 대출 기록'을 '학번'이라는 공통 열을 기준으로 합쳐, 학생별 대출 정보를 만드는 것과 같습니다. (SQL의 JOIN과 유사)
```python
import pandas as pd
df1 = pd.DataFrame({'id': ['A01', 'A02', 'A03'], '이름': ['김파이', '이판다', '박넘파']})
df2 = pd.DataFrame({'id': ['A01', 'A02', 'A04'], '과목': ['수학', '영어', '과학']})

# 공통 'id'를 기준으로 병합 (inner: 양쪽 모두에 id가 있는 경우만)
inner_join = pd.merge(df1, df2, on='id', how='inner')
print("Inner Join:\n", inner_join)

# df1을 기준으로 병합 (left: df1의 모든 id를 유지)
left_join = pd.merge(df1, df2, on='id', how='left')
print("\nLeft Join:\n", left_join)
```

### 2. 데이터 연결: `pd.concat()`
> `concat`은 '1반 학생 명단' 아래에 '2반 학생 명단'을 그대로 이어 붙여 전체 학생 명단을 만드는 것과 같습니다.
```python
df1 = pd.DataFrame({'이름': ['김파이', '이판다'], '나이': [20, 22]})
df2 = pd.DataFrame({'이름': ['박넘파', '최사이'], '나이': [25, 21]})

concatenated_df = pd.concat([df1, df2], ignore_index=True) # 기존 인덱스 무시
print("\nConcatenated DataFrame:\n", concatenated_df)
```

### 3. 데이터 재구조화: `pivot_table()`
> `pivot_table`은 '일자별, 상품별 판매 기록'이라는 긴 목록을 '일자는 행, 상품은 열, 값은 판매량'인 엑셀 보고서 형태로 바꾸는 것과 같습니다.
```python
data = {'날짜': ['2023-01-01', '2023-01-01', '2023-01-02', '2023-01-02'],
        '상품': ['사과', '바나나', '사과', '바나나'],
        '판매량': [100, 80, 120, 90]}
df_sales = pd.DataFrame(data)

pivot = df_sales.pivot_table(index='날짜', columns='상품', values='판매량')
print("\nPivot Table:\n", pivot)
```
---
---

### 📚 5주차 마무리 및 다음 단계
이번 주 우리는 데이터 과학의 양대 산맥인 **NumPy**와 **Pandas**의 기본적인 사용법을 마스터했습니다. 이제 여러분은 대부분의 정형 데이터를 자유자재로 불러오고, 원하는 형태로 가공하고, 기본적인 통계를 파악할 수 있는 강력한 무기를 갖추게 되었습니다.

다음 시간에는 이어서 데이터를 시각적으로 탐색하는 **Matplotlib**과 **Seaborn**을 배우고, 드디어 첫 **머신러닝 모델**을 만들어보는 흥미로운 여정을 시작합니다.

**➡️ 다음 시간: [Part 6: 머신러닝 모델링과 평가](./part_6_machine_learning.md)** 